@@ -242,19 +242,28 @@ class Calib():
                                         np.logical_or(point2D.y < -self.height, point2D.y > 2*self.height))
         return Point2D(np.where(excluded_points, point2D, self.distort(point2D)))
 
-    def project_2D_to_3D(self, point2D: Point2D, Z: float) -> Point3D:
-        """ Using the calib object, project a 2D point in the 3D image space.
+    def project_2D_to_3D(self, point2D: Point2D, X: float=None, Y: float=None, Z: float=None) -> Point3D:
+        """ Using the calib object, project a 2D point in the 3D image space
+            given one of it's 3D coordinates (X,Y or Z). One and only one
+            coordinate must be given.
             Args:
                 point2D (Point2D): the 2D point to be projected
+                X (float): the X coordinate of the 3D point
+                Y (float): the Y coordinate of the 3D point
                 Z (float): the Z coordinate of the 3D point
             Returns:
-                The point in the 3D world for which the z=`Z` and that projects on `point2D`.
+                The point in the 3D world that projects on `point2D` and for
+                which the given coordinates is given.
         """
+        v = [X,Y,Z]
+        assert sum([1 for x in v if x is None]) == 2
         assert isinstance(point2D, Point2D), "Wrong argument type '{}'. Expected {}".format(type(point2D), Point2D)
         point2D = self.rectify(point2D)
         X = Point3D(self.Pinv @ point2D.H)
         d = (X - self.C)
-        return line_plane_intersection(self.C, d, Point3D(0, 0, Z), np.array([[0, 0, 1]]).T)
+        P = np.nan_to_num(Point3D(*v), 0)
+        v = np.array([[0 if x is None else 1 for x in v]]).T
+        return line_plane_intersection(self.C, d, P, v)
 
     def distort(self, point2D: Point2D) -> Point2D:
         """ Applies lens distortions to the given `point2D`.
@@ -315,7 +324,7 @@ class Calib():
     def rotate(self, angle) -> 'Calib':
         if angle == 0:
             return self
-        A, new_width, new_height = compute_rotate(self.width, self.height, angle)
+        A, new_width, new_height = compute_rotate(self.width, self.height, angle, degrees=True)
         return self.update(K=A@self.K, width=new_width, height=new_height)
 
     def compute_length2D(self, length3D: float, point3D: Point3D) -> np.ndarray:
@@ -352,9 +361,10 @@ def line_plane_intersection(C: Point3D, d, P: Point3D, n) -> Point3D:
     dist = ((P-C).T @ n) / dot  # Distance between plane z=Z and camera
     return Point3D(C + dist.T*d)
 
-def compute_rotate(width, height, angle):
+def compute_rotate(width, height, angle, degrees=True):
     """ Computes rotation matrix and new width and height for a rotation of angle degrees of a widthxheight image.
     """
+    assert degrees, "Angle in gradient is not implemented (yet)"
     # Convert the OpenCV 3x2 rotation matrix to 3x3
     R = np.eye(3)
     R[0:2,:] = cv2.getRotationMatrix2D((width/2, height/2), angle, 1.0)
@@ -398,29 +408,33 @@ def parameters_to_affine_transform(angle: float, x_slice: slice, y_slice: slice,
     output_shape: tuple, do_flip: bool=False):
     """ Compute the affine transformation matrix that correspond to a
         - horizontal flip if `do_flip` is `True`, followed by a
-        - rotation of `angle` degrees around image center, followed by a
+        - rotation of `angle` degrees around output image center, followed by a
         - crop defined by `x_slice` and `y_slice`, followed by a
         - scale to recover `output_shape`.
     """
-    assert not do_flip, "There is a bug with random flip"
+    # Rotation
     R = np.eye(3)
-    center = ((y_slice.start + y_slice.stop)/2, (x_slice.start + x_slice.stop)/2)
+    center = ((x_slice.start + x_slice.stop)/2, (y_slice.start + y_slice.stop)/2)
     R[0:2,:] = cv2.getRotationMatrix2D(center, angle, 1.0)
 
+    # Crop
     x0 = x_slice.start
     y0 = y_slice.start
     width = x_slice.stop - x_slice.start
     height = y_slice.stop - y_slice.start
-    T = np.array([[1, 0,-x0], [0, 1,-y0], [0, 0, 1]])
+    C = np.array([[1, 0,-x0], [0, 1,-y0], [0, 0, 1]])
 
+    # Scale
     sx = output_shape[0]/width
     sy = output_shape[1]/height
     S = np.array([[sx, 0, 0], [0, sy, 0], [0, 0, 1]])
 
+    # Random Flip
+    assert not do_flip, "There is a bug with random flip"
     f = np.random.randint(0,2)*2-1 if do_flip else 1 # random sample in {-1,1}
     F = np.array([[f, 0, 0], [0, 1, 0], [0, 0, 1]])
 
-    return S@T@R@F
+    return S@C@R@F
 
 
 def compute_rotation_matrix(point3D: Point3D, camera3D: Point3D):
